@@ -24,8 +24,9 @@ import gobject
 import ibus
 from ibus import keysyms
 from ibus import modifier
-import sys, os, os.path
+import sys, os, os.path, time
 import skk
+import nicola
 
 from gettext import dgettext
 _  = lambda a : dgettext("ibus-skk", a)
@@ -61,9 +62,6 @@ class CandidateSelector(skk.CandidateSelector):
             elif not move_over_pages:
                 self.__lookup_table.set_cursor_pos(self.index() -
                                                    self.pagination_start)
-            else:
-                self.__lookup_table.page_down()
-                self.__lookup_table.set_cursor_pos_in_current_page(0)
         return self.candidate()
 
     def previous_candidate(self, move_over_pages=True):
@@ -74,9 +72,6 @@ class CandidateSelector(skk.CandidateSelector):
             elif not move_over_pages:
                 self.__lookup_table.set_cursor_pos(self.index() -
                                                    self.pagination_start)
-            else:
-                self.__lookup_table.page_up()
-                self.__lookup_table.set_cursor_pos_in_current_page(0)
         return self.candidate()
 
     __emacsclient_paths = ('/usr/bin/emacsclient',
@@ -115,6 +110,13 @@ class CandidateSelector(skk.CandidateSelector):
         else:
             raise IndexError('invalid key position %d' % pos)
 
+    def pos_to_index(self, pos):
+        if self.__lookup_table.set_cursor_pos_in_current_page(pos):
+            index = self.__lookup_table.get_cursor_pos()
+            return self.__pagination_start + index
+        else:
+            raise IndexError('invalid key position %d' % pos)
+
 class Engine(ibus.EngineBase):
     config = None
     sysdict = None
@@ -148,11 +150,9 @@ class Engine(ibus.EngineBase):
         super(Engine, self).__init__(bus, object_path)
         self.__is_invalidate = False
         labels = [ibus.Text(c.upper() + u':') for c in self.__select_keys]
-        page_size = self.config.get_value('page_size',
-                                          skk.CandidateSelector.PAGE_SIZE)
+        page_size = self.config.get_value('page_size')
         pagination_start = \
-            self.config.get_value('pagination_start',
-                                  skk.CandidateSelector.PAGINATION_START)
+            self.config.get_value('pagination_start')
         self.__lookup_table = ibus.LookupTable(page_size=page_size,
                                                round=False,
                                                labels=labels)
@@ -166,22 +166,23 @@ class Engine(ibus.EngineBase):
         usrdict = skk.UsrDict(self.config.usrdict_path)
         self.__skk = skk.Context(usrdict, self.sysdict,
                                  self.__candidate_selector)
-        self.__skk.kutouten_type = self.config.get_value('period_style',
-                                                         skk.KUTOUTEN_JP)
+        self.__skk.kutouten_type = self.config.get_value('period_style')
         auto_start_henkan_keywords = \
-            self.config.get_value('auto_start_henkan_keywords',
-                                  ''.join(skk.AUTO_START_HENKAN_KEYWORDS))
+            self.config.get_value('auto_start_henkan_keywords')
         self.__skk.auto_start_henkan_keywords = \
             list(iter(auto_start_henkan_keywords))
-        self.__skk.rom_kana_rule = self.config.get_value('rom_kana_rule',
-                                                         skk.ROM_KANA_NORMAL)
+        self.__skk.rom_kana_rule = self.config.get_value('rom_kana_rule')
+        self.__skk.egg_like_newline = self.config.get_value('egg_like_newline')
+        self.__skk.direct_input_on_latin = \
+            self.config.get_value('direct_input_on_latin')
         self.__initial_input_mode = \
-            self.config.get_value('initial_input_mode',
-                                  skk.INPUT_MODE_HIRAGANA)
+            self.config.get_value('initial_input_mode')
         self.__skk.translated_strings['dict-edit-prompt'] =\
             _(u'DictEdit').decode('UTF-8')
         self.__skk.translated_strings['kuten-prompt'] =\
             _(u'Kuten([MM]KKTT) ').decode('UTF-8')
+        self.__skk.custom_rom_kana_rule = \
+            self.config.get_value('custom_rom_kana_rule')
         self.__skk.reset()
         self.__skk.activate_input_mode(self.__initial_input_mode)
         self.__prop_dict = dict()
@@ -189,6 +190,12 @@ class Engine(ibus.EngineBase):
         self.__input_mode = skk.INPUT_MODE_NONE
         self.__update_input_mode()
         self.__suspended_mode = None
+        if self.config.get_value('use_nicola'):
+            self.__nicola = nicola.Nicola(time_func=time.time)
+            self.__nicola_handler = None
+        else:
+            self.__nicola = None
+
 
     def __init_props(self):
         skk_props = ibus.PropList()
@@ -225,8 +232,7 @@ class Engine(ibus.EngineBase):
         skk_props.append(input_mode_prop)
 
         skk_props.append(ibus.Property(key=u"setup",
-                                         icon=u"ibus-setup",
-                                         tooltip=_(u"Configure SKK")))
+                                       tooltip=_(u"Configure SKK")))
 
         return skk_props
 
@@ -265,7 +271,8 @@ class Engine(ibus.EngineBase):
                 self.__skk.next_candidate(False)
                 self.__update()
                 return True
-            elif self.__candidate_selector.lookup_table_visible():
+            elif state & modifier.CONTROL_MASK == 0 and \
+                    self.__candidate_selector.lookup_table_visible():
                 try:
                     index = self.__candidate_selector.\
                         key_to_index(unichr(keyval).lower())
@@ -289,25 +296,52 @@ class Engine(ibus.EngineBase):
             keychr = u'escape'
         elif keyval == keysyms.BackSpace:
             keychr = u'backspace'
+        elif self.__nicola and keyval == keysyms.Muhenkan:
+            keychr = u'lshift'
+        elif self.__nicola and keyval == keysyms.Henkan:
+            keychr = u'rshift'
         else:
             keychr = unichr(keyval)
             if 0x20 > ord(keychr) or ord(keychr) > 0x7E:
-                return False
-        if keychr.isalpha():
-            keychr = keychr.lower()
-        if state & modifier.SHIFT_MASK:
-            keychr = u'shift+' + keychr
+                # If the pre-edit buffer is visible, always handle key events:
+                # http://github.com/ueno/ibus-skk/issues/#issue/5
+                return len(self.__skk.preedit) > 0
         if state & modifier.CONTROL_MASK:
-            keychr = u'ctrl+' + keychr
+            # Some systems return 'J' if ctrl:nocaps xkb option is
+            # enabled and the user press CapsLock + 'j':
+            # http://github.com/ueno/ibus-skk/issues/#issue/22
+            keychr = u'ctrl+' + keychr.lower()
+        elif self.__nicola:
+            if keychr in (u'lshift', u'rshift') or len(keychr) == 1:
+                self.__nicola.queue(keychr)
+            else:
+                return self.__skk_press_key(keychr)
+            self.__nicola_dispatch()
+            return True
+        return self.__skk_press_key(keychr)
+
+    def __nicola_dispatch(self, is_timer=False):
+        if self.__nicola_handler:
+            gobject.source_remove(self.__nicola_handler)
+        result = self.__nicola.dispatch()
+        for keychr in result.output:
+            self.__skk_press_key(u'nicola+' + keychr)
+        self.__nicola_handler = gobject.timeout_add(int(result.wait * 1000),
+                                                    self.__nicola_dispatch,
+                                                    True)
+
+    def __skk_press_key(self, keychr):
         handled, output = self.__skk.press_key(keychr)
+        if output:
+            self.commit_text(ibus.Text(output))
         if handled:
-            if output:
-                self.commit_text(ibus.Text(output))
             gobject.idle_add(self.__skk.usrdict.save,
                              priority = gobject.PRIORITY_LOW)
             self.__update()
             return True
-        return False
+        # If the pre-edit buffer is visible, always handle key events:
+        # http://github.com/ueno/ibus-skk/issues/#issue/5
+        return len(self.__skk.preedit) > 0
 
     def __invalidate(self):
         if self.__is_invalidate:
@@ -318,40 +352,58 @@ class Engine(ibus.EngineBase):
     def page_up(self):
         if self.__lookup_table.page_up():
             self.page_up_lookup_table()
+            self.__candidate_selector.previous_candidate(True)
+            self.__update()
             return True
         return False
 
     def page_down(self):
         if self.__lookup_table.page_down():
             self.page_down_lookup_table()
+            self.__candidate_selector.next_candidate(True)
+            self.__update()
             return True
         return False
 
     def cursor_up(self):
         if self.__lookup_table.cursor_up():
             self.cursor_up_lookup_table()
+            self.__candidate_selector.previous_candidate(False)
+            self.__update()
             return True
         return False
 
     def cursor_down(self):
         if self.__lookup_table.cursor_down():
             self.cursor_down_lookup_table()
+            self.__candidate_selector.next_candidate(False)
+            self.__update()
             return True
         return False
 
+    def candidate_clicked(self, index, button, state):
+        try:
+            index = self.__candidate_selector.pos_to_index(index)
+            handled, output = self.__skk.select_candidate(index)
+            if handled:
+                if output:
+                    self.commit_text(ibus.Text(output))
+                gobject.idle_add(self.__skk.usrdict.save,
+                                 priority = gobject.PRIORITY_LOW)
+                self.__lookup_table.clean()
+                self.__update()
+        except IndexError:
+            pass
+        
     def __possibly_update_config(self):
         if self.__skk.usrdict.path != self.config.usrdict_path:
             self.__skk.usrdict = skk.UsrDict(self.config.usrdict_path)
-        self.__skk.kutouten_type = self.config.get_value('period_style',
-                                                         skk.KUTOUTEN_JP)
-        self.__skk.rom_kana_rule = self.config.get_value('rom_kana_rule',
-                                                         skk.ROM_KANA_NORMAL)
+        self.__skk.kutouten_type = self.config.get_value('period_style')
+        self.__skk.rom_kana_rule = self.config.get_value('rom_kana_rule')
         self.__initial_input_mode = \
-            self.config.get_value('initial_input_mode',
-                                  skk.INPUT_MODE_HIRAGANA)
+            self.config.get_value('initial_input_mode')
         self.__skk.auto_start_henkan_keywords = \
-            list(iter(self.config.get_value('auto_start_henkan_keywords',
-                                            ''.join(skk.AUTO_START_HENKAN_KEYWORDS))))
+            list(iter(self.config.get_value('auto_start_henkan_keywords')))
 
     # ABBREV_CURSOR_COLOR = (65, 105, 225)
     # INPUT_MODE_CURSOR_COLORS = {
@@ -401,7 +453,7 @@ class Engine(ibus.EngineBase):
         self.update_preedit_text(ibus.Text(preedit, attrs),
                                  len(preedit), len(preedit) > 0)
         visible = self.__candidate_selector.lookup_table_visible()
-        if self.config.get_value('show_annotation', True):
+        if self.config.get_value('show_annotation'):
             candidate = self.__candidate_selector.candidate()
             annotation = candidate[1] if candidate else None
             self.update_auxiliary_text(ibus.Text(annotation or u''), visible)
