@@ -4,6 +4,8 @@ from __future__ import with_statement
 import unittest
 import os, os.path
 import skk
+import nicola
+from ibus import modifier
 
 class TestSKK(unittest.TestCase):
     def setUp(self):
@@ -29,8 +31,11 @@ class TestSKK(unittest.TestCase):
                         if line.startswith(';; okuri-nasi'):
                             tp.write(u'#/# /#0月#0日/#1／#1/#1月#1日/\n'.encode('EUC-JP'))
                             tp.write(u'#ひき /#1匹/#3匹/#0匹/#2匹/\n'.encode('EUC-JP'))
+                            tp.write(u'>し /氏/\n'.encode('EUC-JP'))
                         if line.startswith('greek '):
                             tp.write(u'request /リクエスト/\n'.encode('EUC-JP'))
+                        if line.startswith(u'ちょう '.encode('EUC-JP')):
+                            tp.write(u'ちょう> /超/\n'.encode('EUC-JP'))
 
         self.__skk = skk.Context(usrdict=skk.UsrDict(usrdict_path),
                                  sysdict=skk.SysDict(sysdict_path),
@@ -105,7 +110,7 @@ class TestSKK(unittest.TestCase):
         self.assertEqual(self.__skk.preedit, u'')
         self.assertEqual(self.__skk.input_mode, skk.INPUT_MODE_HIRAGANA)
         # HIRAGANA to WIDE-LATIN
-        handled, output = self.__skk.press_key(u'shift+l')
+        handled, output = self.__skk.press_key(u'L')
         self.assert_(handled)
         self.assertEqual(output, u'')
         self.assertEqual(self.__skk.conv_state, skk.CONV_STATE_NONE)
@@ -181,7 +186,7 @@ class TestSKK(unittest.TestCase):
     def testhiraganakatakana(self):
         self.__skk.reset()
         self.__skk.activate_input_mode(skk.INPUT_MODE_HIRAGANA)
-        self.__skk.press_key(u'shift+a')
+        self.__skk.press_key(u'A')
         self.assertEqual(self.__skk.conv_state, skk.CONV_STATE_START)
         self.__skk.press_key(u'i')
         self.assertEqual(self.__skk.press_key(u'q'), (True, u'アイ'))
@@ -191,7 +196,7 @@ class TestSKK(unittest.TestCase):
         self.assertEqual(self.__skk.input_mode, skk.INPUT_MODE_KATAKANA)
         self.assertEqual(self.__skk.conv_state, skk.CONV_STATE_NONE)
         self.assertEqual(self.__skk.preedit, u'')
-        self.__skk.press_key(u'shift+a')
+        self.__skk.press_key(u'A')
         self.assertEqual(self.__skk.conv_state, skk.CONV_STATE_START)
         self.__skk.press_key(u'i')
         self.assertEqual(self.__skk.press_key(u'q'), (True, u'あい'))
@@ -201,21 +206,21 @@ class TestSKK(unittest.TestCase):
         # う゛-> ヴ
         self.__skk.reset()
         self.__skk.activate_input_mode(skk.INPUT_MODE_HIRAGANA)
-        self.__skk.press_key(u'shift+v')
+        self.__skk.press_key(u'V')
         self.__skk.press_key(u'u')
         self.assertEqual(self.__skk.preedit, u'▽う゛')
         self.assertEqual(self.__skk.press_key(u'q'), (True, u'ヴ'))
         # ヴ -> う゛
         self.__skk.reset()
         self.__skk.activate_input_mode(skk.INPUT_MODE_KATAKANA)
-        self.__skk.press_key(u'shift+v')
+        self.__skk.press_key(u'V')
         self.__skk.press_key(u'u')
         self.assertEqual(self.__skk.preedit, u'▽ヴ')
         self.assertEqual(self.__skk.press_key(u'q'), (True, u'う゛'))
         # ▽nq -> ン
         self.__skk.reset()
         self.__skk.activate_input_mode(skk.INPUT_MODE_HIRAGANA)
-        self.__skk.press_key(u'shift+q')
+        self.__skk.press_key(u'Q')
         self.__skk.press_key(u'n')
         handled, output = self.__skk.press_key(u'q')
         self.assertTrue(handled)
@@ -224,7 +229,7 @@ class TestSKK(unittest.TestCase):
     def testokurinasi(self):
         self.__skk.reset()
         self.__skk.activate_input_mode(skk.INPUT_MODE_HIRAGANA)
-        self.assertEqual(self.__skk.press_key(u'shift+a'), (True, u''))
+        self.assertEqual(self.__skk.press_key(u'A'), (True, u''))
         self.assertEqual(self.__skk.preedit, u'▽あ')
         self.assertEqual(self.__skk.press_key(u'i'), (True, u''))
         self.assertEqual(self.__skk.preedit, u'▽あい')
@@ -232,16 +237,60 @@ class TestSKK(unittest.TestCase):
         self.assertEqual(self.__skk.preedit, u'▼愛')
         self.__skk.press_key(u' ')
         self.assertEqual(self.__skk.preedit, u'▼哀')
+        # NA -> ▽な, NAN -> ▽な*n, NANA -> [DictEdit] な*んあ
+        self.__skk.reset()
+        self.__skk.activate_input_mode(skk.INPUT_MODE_HIRAGANA)
+        self.__skk.press_key(u'N')
+        self.__skk.press_key(u'A')
+        self.assertEqual(self.__skk.preedit, u'▽な')
+        self.__skk.press_key(u'N')
+        self.assertEqual(self.__skk.preedit, u'▽な*n')
+        self.__skk.press_key(u'A')
+        self.assertEqual(self.__skk.preedit, u'[DictEdit] な*んあ ')
+        self.__skk.press_key(u'ctrl+g')
+        self.__skk.press_key(u'return')
+        # NA -> ▽な, NAN -> ▽な*n, NANa -> [DictEdit] な*な
+        self.__skk.press_key(u'N')
+        self.__skk.press_key(u'A')
+        self.assertEqual(self.__skk.preedit, u'▽な')
+        self.__skk.press_key(u'N')
+        self.assertEqual(self.__skk.preedit, u'▽な*n')
+        self.__skk.press_key(u'a')
+        self.assertEqual(self.__skk.preedit, u'[DictEdit] な*な ')
+
+    def testegglikenewline(self):
+        self.__skk.reset()
+        self.__skk.activate_input_mode(skk.INPUT_MODE_HIRAGANA)
+        # CONV_STATE_START
+        self.__skk.press_key(u'A')
+        self.__skk.press_key(u'i')
+        self.__skk.egg_like_newline = False
+        self.assertEqual(self.__skk.press_key(u'return'), (True, u'あい\n'))
+        self.__skk.press_key(u'A')
+        self.__skk.press_key(u'i')
+        self.__skk.egg_like_newline = True
+        self.assertEqual(self.__skk.press_key(u'return'), (True, u'あい'))
+        # CONV_STATE_SELECT
+        self.__skk.press_key(u'A')
+        self.__skk.press_key(u'i')
+        self.__skk.press_key(u' ')
+        self.__skk.egg_like_newline = False
+        self.assertEqual(self.__skk.press_key(u'return'), (True, u'愛\n'))
+        self.__skk.press_key(u'A')
+        self.__skk.press_key(u'i')
+        self.__skk.press_key(u' ')
+        self.__skk.egg_like_newline = True
+        self.assertEqual(self.__skk.press_key(u'return'), (True, u'愛'))
 
     def testokuriari(self):
         self.__skk.reset()
         self.__skk.activate_input_mode(skk.INPUT_MODE_HIRAGANA)
-        self.__skk.press_key(u'shift+k')
+        self.__skk.press_key(u'K')
         self.__skk.press_key(u'a')
         self.__skk.press_key(u'n')
         self.__skk.press_key(u'g')
         self.__skk.press_key(u'a')
-        self.__skk.press_key(u'shift+e')
+        self.__skk.press_key(u'E')
         self.assertEqual(self.__skk.preedit, u'▼考え')
         self.assertEqual(self.__skk.press_key(u'r'), (True, u'考え'))
         self.assertEqual(self.__skk.preedit, u'r')
@@ -249,36 +298,73 @@ class TestSKK(unittest.TestCase):
 
         self.__skk.reset()
         self.__skk.activate_input_mode(skk.INPUT_MODE_HIRAGANA)
-        self.__skk.press_key(u'shift+h')
+        self.__skk.press_key(u'H')
         self.__skk.press_key(u'a')
-        self.__skk.press_key(u'shift+z')
+        self.__skk.press_key(u'Z')
         self.assertEqual(self.__skk.preedit, u'▽は*z')
         self.__skk.press_key(u'u')
         self.assertEqual(self.__skk.preedit, u'▼恥ず')
 
         self.__skk.reset()
         self.__skk.activate_input_mode(skk.INPUT_MODE_HIRAGANA)
-        self.__skk.press_key(u'shift+t')
+        self.__skk.press_key(u'T')
         self.__skk.press_key(u'u')
         self.__skk.press_key(u'k')
         self.__skk.press_key(u'a')
-        self.__skk.press_key(u'shift+t')
+        self.__skk.press_key(u'T')
         self.__skk.press_key(u't')
         self.assertEqual(self.__skk.preedit, u'▽つか*っt')
 
         # Debian Bug#591052
         self.__skk.reset()
         self.__skk.activate_input_mode(skk.INPUT_MODE_HIRAGANA)
-        self.__skk.press_key(u'shift+k')
+        self.__skk.press_key(u'K')
         self.__skk.press_key(u'a')
         self.__skk.press_key(u'n')
-        self.__skk.press_key(u'shift+j')
+        self.__skk.press_key(u'J')
         self.assertEqual(self.__skk.preedit, u'▽かん*j')
 
+        # Issue#10
+        self.__skk.debug = True
+        self.__skk.reset()
+        self.__skk.activate_input_mode(skk.INPUT_MODE_HIRAGANA)
+        self.__skk.press_key(u'F')
+        self.__skk.press_key(u'u')
+        self.__skk.press_key(u'N')
+        self.__skk.press_key(u'd')
+        self.__skk.press_key(u'a')
+        self.assertEqual(self.__skk.preedit, u'▼踏んだ')
+        self.__skk.debug = False
+
+        # Issue#19
+        self.__skk.reset()
+        self.__skk.activate_input_mode(skk.INPUT_MODE_HIRAGANA)
+        self.__skk.press_key(u'A')
+        self.__skk.press_key(u'z')
+        self.__skk.press_key(u'u')
+        self.__skk.press_key(u'm')
+        self.__skk.press_key(u'a')
+        self.__skk.press_key(u' ')
+        self.assertEqual(self.__skk.preedit, u'▼東')
+        self.__skk.press_key(u'>')
+        self.assertEqual(self.__skk.preedit, u'▽>')
+        self.__skk.press_key(u's')
+        self.__skk.press_key(u'h')
+        self.__skk.press_key(u'i')
+        self.__skk.press_key(u' ')
+        self.assertEqual(self.__skk.preedit, u'▼氏')
+        self.__skk.kakutei()
+        self.__skk.press_key(u'T')
+        self.__skk.press_key(u'y')
+        self.__skk.press_key(u'o')
+        self.__skk.press_key(u'u')
+        self.__skk.press_key(u'>')
+        self.assertEqual(self.__skk.preedit, u'▼超')
+
     def testcompletion(self):
         self.__skk.reset()
         self.__skk.activate_input_mode(skk.INPUT_MODE_HIRAGANA)
-        self.__skk.press_key(u'shift+a')
+        self.__skk.press_key(u'A')
         self.assertEqual(self.__skk.preedit, u'▽あ')
         self.__skk.press_key(u'\t')
         self.assertEqual(self.__skk.preedit, u'▽あい')
@@ -286,14 +372,14 @@ class TestSKK(unittest.TestCase):
         self.assertEqual(self.__skk.preedit, u'▽あいさつ')
         self.__skk.press_key(u' ')
         self.__skk.kakutei()
-        self.__skk.press_key(u'shift+a')
+        self.__skk.press_key(u'A')
         self.assertEqual(self.__skk.preedit, u'▽あ')
         self.__skk.press_key(u'\t')
         self.assertEqual(self.__skk.preedit, u'▽あいさつ')
         self.__skk.press_key(u' ')
         self.__skk.kakutei()
         self.assertEqual(self.__skk.preedit, u'')
-        self.__skk.press_key(u'shift+a')
+        self.__skk.press_key(u'A')
         self.assertEqual(self.__skk.preedit, u'▽あ')
         self.__skk.press_key(u'ctrl+i')
         self.assertEqual(self.__skk.preedit, u'▽あいさつ')
@@ -301,7 +387,7 @@ class TestSKK(unittest.TestCase):
     def testautoconvesion(self):
         self.__skk.reset()
         self.__skk.activate_input_mode(skk.INPUT_MODE_HIRAGANA)
-        self.__skk.press_key(u'shift+a')
+        self.__skk.press_key(u'A')
         self.assertEqual(self.__skk.preedit, u'▽あ')
         self.__skk.press_key(u'i')
         self.assertEqual(self.__skk.preedit, u'▽あい')
@@ -311,7 +397,7 @@ class TestSKK(unittest.TestCase):
         self.assertEqual(self.__skk.preedit, u'▼哀、')
         self.__skk.reset()
         self.__skk.activate_input_mode(skk.INPUT_MODE_HIRAGANA)
-        self.__skk.press_key(u'shift+a')
+        self.__skk.press_key(u'A')
         self.__skk.press_key(u'i')
         self.__skk.press_key(u'w')
         self.__skk.press_key(u'o')
@@ -320,7 +406,7 @@ class TestSKK(unittest.TestCase):
     def testdelete(self):
         self.__skk.reset()
         self.__skk.activate_input_mode(skk.INPUT_MODE_HIRAGANA)
-        self.__skk.press_key(u'shift+a')
+        self.__skk.press_key(u'A')
         self.assertEqual(self.__skk.preedit, u'▽あ')
         handled, output = self.__skk.press_key(u'backspace')
         self.assertTrue(handled)
@@ -331,7 +417,7 @@ class TestSKK(unittest.TestCase):
 
         self.__skk.reset()
         self.__skk.activate_input_mode(skk.INPUT_MODE_HIRAGANA)
-        self.__skk.press_key(u'shift+a')
+        self.__skk.press_key(u'A')
         self.assertEqual(self.__skk.preedit, u'▽あ')
         self.__skk.press_key(u'i')
         self.__skk.press_key(u's')
@@ -351,7 +437,7 @@ class TestSKK(unittest.TestCase):
     def testdeleteshortcut(self):
         self.__skk.reset()
         self.__skk.activate_input_mode(skk.INPUT_MODE_HIRAGANA)
-        self.__skk.press_key(u'shift+a')
+        self.__skk.press_key(u'A')
         self.assertEqual(self.__skk.preedit, u'▽あ')
         handled, output = self.__skk.press_key(u'ctrl+h')
         self.assertTrue(handled)
@@ -362,7 +448,7 @@ class TestSKK(unittest.TestCase):
 
         self.__skk.reset()
         self.__skk.activate_input_mode(skk.INPUT_MODE_HIRAGANA)
-        self.__skk.press_key(u'shift+a')
+        self.__skk.press_key(u'A')
         self.assertEqual(self.__skk.preedit, u'▽あ')
         self.__skk.press_key(u'i')
         self.__skk.press_key(u's')
@@ -382,7 +468,7 @@ class TestSKK(unittest.TestCase):
     def testnumeric(self):
         self.__skk.reset()
         self.__skk.activate_input_mode(skk.INPUT_MODE_HIRAGANA)
-        self.__skk.press_key(u'shift+q')
+        self.__skk.press_key(u'Q')
         self.assertEqual(self.__skk.preedit, u'▽')
         self.__skk.press_key(u'5')
         self.__skk.press_key(u'/')
@@ -392,7 +478,7 @@ class TestSKK(unittest.TestCase):
 
         self.__skk.reset()
         self.__skk.activate_input_mode(skk.INPUT_MODE_HIRAGANA)
-        self.__skk.press_key(u'shift+q')
+        self.__skk.press_key(u'Q')
         self.__skk.press_key(u'5')
         self.__skk.press_key(u'h')
         self.__skk.press_key(u'i')
@@ -405,7 +491,7 @@ class TestSKK(unittest.TestCase):
 
         self.__skk.reset()
         self.__skk.activate_input_mode(skk.INPUT_MODE_HIRAGANA)
-        self.__skk.press_key(u'shift+q')
+        self.__skk.press_key(u'Q')
         self.assertEqual(self.__skk.preedit, u'▽')
         self.__skk.press_key(u'5')
         self.__skk.press_key(u'0')
@@ -443,7 +529,7 @@ class TestSKK(unittest.TestCase):
     def testdictedit(self):
         self.__skk.reset()
         self.__skk.activate_input_mode(skk.INPUT_MODE_HIRAGANA)
-        self.__skk.press_key(u'shift+k')
+        self.__skk.press_key(u'K')
         self.__skk.press_key(u'a')
         self.__skk.press_key(u'p')
         self.__skk.press_key(u'a')
@@ -452,18 +538,18 @@ class TestSKK(unittest.TestCase):
         self.__skk.press_key(u'a')
         self.assertEqual(self.__skk.preedit, u'[DictEdit] かぱ あ')
         self.__skk.press_key(u'backspace')
-        self.__skk.press_key(u'shift+a')
+        self.__skk.press_key(u'A')
         self.__skk.press_key(u'ctrl+j')
         self.assertEqual(self.__skk.preedit, u'[DictEdit] かぱ あ')
         self.__skk.press_key(u'backspace')
-        self.__skk.press_key(u'shift+k')
+        self.__skk.press_key(u'K')
         self.__skk.press_key(u'a')
         self.__skk.press_key(u' ')
         self.__skk.press_key(u'ctrl+g')
         self.assertEqual(self.__skk.preedit, u'[DictEdit] かぱ ▽か')
         self.__skk.press_key(u'return')
         self.__skk.press_key(u'backspace')
-        self.__skk.press_key(u'shift+k')
+        self.__skk.press_key(u'K')
         self.__skk.press_key(u'a')
         self.__skk.press_key(u'p')
         self.__skk.press_key(u'a')
@@ -478,12 +564,12 @@ class TestSKK(unittest.TestCase):
         self.assertEqual(self.__skk.preedit, u'▽かぱ')
         self.__skk.press_key(u' ')
         self.assertEqual(self.__skk.preedit, u'[DictEdit] かぱ ')
-        self.__skk.press_key(u'shift+k')
+        self.__skk.press_key(u'K')
         self.__skk.press_key(u'a')
         self.assertEqual(self.__skk.preedit, u'[DictEdit] かぱ ▽か')
         self.__skk.press_key(u' ')
         self.assertEqual(self.__skk.preedit, u'[DictEdit] かぱ ▼下')
-        self.__skk.press_key(u'shift+h')
+        self.__skk.press_key(u'H')
         self.__skk.press_key(u'a')
         self.__skk.press_key(u' ')
         self.__skk.press_key(u'return')
@@ -491,16 +577,16 @@ class TestSKK(unittest.TestCase):
         self.__skk.press_key(u'backspace')
         self.assertEqual(self.__skk.preedit, u'[DictEdit] かぱ 下破')
         self.assertEqual(self.__skk.press_key(u'return'), (True, u'下破'))
-        self.__skk.press_key(u'shift+k')
+        self.__skk.press_key(u'K')
         self.__skk.press_key(u'a')
         self.__skk.press_key(u'p')
         self.__skk.press_key(u'a')
         self.__skk.press_key(u' ')
         self.assertEqual(self.__skk.preedit, u'▼下破')
         # Purge "下破" from the user dictionary (Debian Bug#590188).
-        self.__skk.press_key(u'shift+x')
+        self.__skk.press_key(u'X')
         self.assertEqual(self.__skk.preedit, u'')
-        self.__skk.press_key(u'shift+k')
+        self.__skk.press_key(u'K')
         self.__skk.press_key(u'a')
         self.__skk.press_key(u'p')
         self.__skk.press_key(u'a')
@@ -511,7 +597,7 @@ class TestSKK(unittest.TestCase):
         self.__skk.press_key(u'ctrl+g')
         self.__skk.press_key(u'ctrl+g')
         self.assertEqual(self.__skk.preedit, u'')
-        self.__skk.press_key(u'shift+k')
+        self.__skk.press_key(u'K')
         self.__skk.press_key(u'a')
         self.__skk.press_key(u'n')
         self.__skk.press_key(u'j')
@@ -522,19 +608,19 @@ class TestSKK(unittest.TestCase):
         self.__skk.press_key(u'ctrl+g')
         self.assertEqual(self.__skk.preedit, u'▼幹事')
         self.__skk.press_key(u'return')
-        self.__skk.press_key(u'shift+k')
+        self.__skk.press_key(u'K')
         self.__skk.press_key(u'a')
         self.__skk.press_key(u'n')
         self.__skk.press_key(u'g')
         self.__skk.press_key(u'a')
-        self.__skk.press_key(u'shift+e')
+        self.__skk.press_key(u'E')
         self.__skk.press_key(u' ')
         self.assertEqual(self.__skk.preedit, u'[DictEdit] かんが*え ')
         self.__skk.press_key(u'ctrl+g')
         self.assertEqual(self.__skk.preedit, u'▼考え')
         self.__skk.press_key(u'ctrl+g')
         self.assertEqual(self.__skk.preedit, u'▽かんがえ')
-        self.__skk.press_key(u'shift+e')
+        self.__skk.press_key(u'E')
         self.assertEqual(self.__skk.preedit, u'[DictEdit] かんがえ*え ')
         self.__skk.press_key(u'ctrl+g')
         self.assertEqual(self.__skk.preedit, u'▽かんがええ')
@@ -543,7 +629,7 @@ class TestSKK(unittest.TestCase):
         self.__skk.press_key(u'ctrl+h')
         self.assertEqual(self.__skk.preedit, u'▽かんが')
         self.__skk.press_key(u'ctrl+g')
-        self.__skk.press_key(u'shift+k')
+        self.__skk.press_key(u'K')
         self.__skk.press_key(u'a')
         self.__skk.press_key(u't')
         self.__skk.press_key(u'a')
@@ -553,7 +639,7 @@ class TestSKK(unittest.TestCase):
         self.__skk.press_key(u'a')
         self.__skk.press_key(u' ')
         self.__skk.press_key(u' ')
-        self.__skk.press_key(u'shift+k')
+        self.__skk.press_key(u'K')
         self.__skk.press_key(u'a')
         self.__skk.press_key(u't')
         self.__skk.press_key(u'a')
@@ -568,13 +654,16 @@ class TestSKK(unittest.TestCase):
         self.__skk.press_key(u'n')
         self.__skk.press_key(u'a')
         self.assertEqual(self.__skk.preedit, u'[DictEdit] かたかな カタカna')
+        # http://github.com/ueno/ibus-skk/issues/#issue/4
+        handled, output = self.__skk.press_key(u'ctrl+m')
+        self.assertEqual(output, u'カタカna')
 
     def testhankaku(self):
         self.__skk.reset()
         self.__skk.activate_input_mode(skk.INPUT_MODE_KATAKANA)
         self.__skk.press_key(u'ctrl+q')
         self.assertEqual(self.__skk.input_mode, skk.INPUT_MODE_HANKAKU_KATAKANA)
-        self.__skk.press_key(u'shift+z')
+        self.__skk.press_key(u'Z')
         self.__skk.press_key(u'e')
         self.__skk.press_key(u'n')
         self.__skk.press_key(u'k')
@@ -585,7 +674,7 @@ class TestSKK(unittest.TestCase):
         handled, output = self.__skk.press_key(u'ctrl+q')
         self.assertTrue(handled)
         self.assertEqual(output, u'ゼンカク')
-        self.__skk.press_key(u'shift+z')
+        self.__skk.press_key(u'Z')
         self.__skk.press_key(u'e')
         self.__skk.press_key(u'n')
         self.__skk.press_key(u'k')
@@ -625,6 +714,33 @@ class TestSKK(unittest.TestCase):
         self.assertEqual(output, u'')
         self.assertEqual(self.__skk.preedit, u'▽]')
 
+        # Ignore "" in abbrev mode (Issue#16).
+        self.__skk.reset();
+        self.__skk.activate_input_mode(skk.INPUT_MODE_HIRAGANA)
+        self.__skk.press_key(u'/')
+        handled, output = self.__skk.press_key(u'(')
+        self.assertTrue(handled)
+        self.assertEqual(output, u'')
+        self.assertEqual(self.__skk.preedit, u'▽(')
+
+        self.__skk.reset();
+        self.__skk.activate_input_mode(skk.INPUT_MODE_HIRAGANA)
+        self.__skk.press_key(u'/')
+        handled, output = self.__skk.press_key(u'A')
+        self.assertTrue(handled)
+        self.assertEqual(output, u'')
+        self.assertEqual(self.__skk.preedit, u'▽A')
+
+        # Convert hankaku to zenkaku with ctrl+q (Issue#17).
+        self.__skk.reset();
+        self.__skk.activate_input_mode(skk.INPUT_MODE_HIRAGANA)
+        self.__skk.press_key(u'/')
+        self.__skk.press_key(u'a')
+        self.__skk.press_key(u'a')
+        handled, output = self.__skk.press_key(u'ctrl+q')
+        self.assertTrue(handled)
+        self.assertEqual(output, u'ａａ')
+
     def testkuten(self):
         self.__skk.reset()
         self.__skk.activate_input_mode(skk.INPUT_MODE_HIRAGANA)
@@ -653,5 +769,97 @@ class TestSKK(unittest.TestCase):
         self.assertTrue(handled)
         self.assertEqual(output, u'＼')
 
+    def testnicola(self):
+        class Counter(object):
+            def __init__(self):
+                self.__counter = 0
+
+            def time(self):
+                return self.__counter
+
+            def incr(self, amount):
+                self.__counter += amount
+
+        t = Counter()
+        n = nicola.Nicola(time_func=t.time)
+        # single key - timeout
+        n.queue('a')
+        result = n.dispatch()
+        self.assertEqual(result.output, tuple())
+        t.incr(0.2)
+        result = n.dispatch()
+        self.assertEqual(result.output, ('a',))
+        # single key - release
+        n.queue('a')
+        n.queue('release+a')
+        result = n.dispatch()
+        self.assertEqual(result.output, ('a',))
+        # single key - overlap
+        n.queue('a')
+        t.incr(0.05)
+        n.queue('b')
+        result = n.dispatch()
+        self.assertEqual(result.output, ('a',))
+        t.incr(0.2)
+        result = n.dispatch()
+        self.assertEqual(result.output, ('b',))
+        # double key - shifted
+        n.queue('a')
+        t.incr(0.01)
+        n.queue('lshift')
+        t.incr(0.2)
+        result = n.dispatch()
+        self.assertEqual(result.output, ('lshift+a',))
+        # double key - shifted reverse
+        n.queue('lshift')
+        t.incr(0.01)
+        n.queue('a')
+        t.incr(0.2)
+        result = n.dispatch()
+        self.assertEqual(result.output, ('lshift+a',))
+        # double key - shifted expired
+        n.queue('a')
+        t.incr(0.06)
+        n.queue('lshift')
+        result = n.dispatch()
+        self.assertNotEqual(result.output, ('lshift+a',))
+        t.incr(0.2)
+        # double key - skk-nicola
+        n.queue('f')
+        t.incr(0.06)
+        n.queue('j')
+        result = n.dispatch()
+        self.assertNotEqual(result.output, ('[fj]',))
+        t.incr(0.2)
+        # double key - skk-nicola (reverse)
+        n.queue('j')
+        t.incr(0.06)
+        n.queue('f')
+        result = n.dispatch()
+        self.assertNotEqual(result.output, ('[fj]',))
+        t.incr(0.2)
+        result = n.dispatch()
+        # triple key t1 <= t2
+        n.queue('a')
+        t.incr(0.01)
+        n.queue('lshift')
+        t.incr(0.02)
+        n.queue('b')
+        result = n.dispatch()
+        self.assertEqual(result.output, ('lshift+a',))
+        result = n.dispatch()
+        self.assertEqual(result.output, tuple())
+        t.incr(0.2)
+        result = n.dispatch()
+        self.assertEqual(result.output, ('b',))
+        # triple key t1 > t2
+        n.queue('a')
+        t.incr(0.02)
+        n.queue('lshift')
+        t.incr(0.01)
+        n.queue('b')
+        result = n.dispatch()
+        self.assertEqual(result.output, ('a', 'lshift+b'))
+
 if __name__ == '__main__':
     unittest.main()
