@@ -1,11 +1,12 @@
 import gtk
 import locale
 import os, sys
-import gettext
 import config
+import gettext
 
-sys.path.insert(0, os.path.join(os.getenv('IBUS_SKK_PKGDATADIR'), 'engine'))
-import skk
+from gettext import dgettext
+_  = lambda a : dgettext("ibus-skk", a)
+N_ = lambda a : a
 
 def get_index_by_value (widget, value):
     model = widget.get_model()
@@ -33,11 +34,16 @@ class PreferencesDialog:
         self.__dialog = self.__builder.get_object('dialog')
 
         self.__usrdict = self.__builder.get_object('usrdict')
-        self.__sysdict_file = self.__builder.get_object('sysdict_file')
-        self.__sysdict = self.__builder.get_object('sysdict')
-        self.__sysdict_skkserv = self.__builder.get_object('sysdict_skkserv')
+        self.__use_skkserv = self.__builder.get_object('use_skkserv')
         self.__skkserv_host = self.__builder.get_object('skkserv_host')
         self.__skkserv_port = self.__builder.get_object('skkserv_port')
+        self.__sysdict = self.__builder.get_object('sysdict')
+        self.__sysdict_liststore = self.__builder.get_object('sysdict_liststore')
+        self.__add_sysdict = self.__builder.get_object('add_sysdict')
+        self.__remove_sysdict = self.__builder.get_object('remove_sysdict')
+        self.__up_sysdict = self.__builder.get_object('up_sysdict')
+        self.__down_sysdict = self.__builder.get_object('down_sysdict')
+        self.__use_mmap = self.__builder.get_object('use_mmap')
         self.__period_style = self.__builder.get_object('period_style')
         self.__page_size = self.__builder.get_object('page_size')
         self.__pagination_start = self.__builder.get_object('pagination_start')
@@ -47,55 +53,58 @@ class PreferencesDialog:
         self.__rom_kana_rule = self.__builder.get_object('rom_kana_rule')
         self.__initial_input_mode = \
             self.__builder.get_object('initial_input_mode')
+        self.__egg_like_newline = self.__builder.get_object('egg_like_newline')
 
         self.__usrdict.set_filename(self.__config.usrdict_path)
-        sysdict_type = self.__config.get_value('sysdict_type', 'file')
-        if sysdict_type == 'file':
-            self.__sysdict_file.set_active(True)
-            self.__sysdict_skkserv.set_active(False)
-        else:
-            self.__sysdict_file.set_active(False)
-            self.__sysdict_skkserv.set_active(True)
+        sysdict_type = self.__config.get_value('sysdict_type')
+        if sysdict_type == 'skkserv':
+            self.__use_skkserv.set_active(True)
         self.__set_sysdict_widgets_sensitivity(sysdict_type)
-        self.__sysdict.set_filename(self.__config.sysdict_path)
+        use_mmap = self.__config.get_value('use_mmap')
+        self.__use_mmap.set_active(use_mmap)
+
+        # sysdict treeview
+        renderer = gtk.CellRendererText()
+        column = gtk.TreeViewColumn("path", renderer, text=0)
+        self.__sysdict.append_column(column)
+        for path in self.__config.sysdict_paths:
+            self.__sysdict_liststore.append((path,))
+
         self.__skkserv_host.set_text(\
-            self.__config.get_value('skkserv_host', skk.SkkServ.HOST))
+            self.__config.get_value('skkserv_host'))
         self.__skkserv_port.set_numeric(True)
         self.__skkserv_port.set_range(0.0, 65535.0)
         self.__skkserv_port.set_value(\
-            int(self.__config.get_value('skkserv_port', skk.SkkServ.PORT)))
+            int(self.__config.get_value('skkserv_port')))
 
         renderer = gtk.CellRendererText()
         self.__period_style.pack_start(renderer)
         self.__period_style.set_attributes(renderer, text=0)
         index = get_index_by_value(\
             self.__period_style,
-            self.__config.get_value('period_style', skk.KUTOUTEN_JP))
+            self.__config.get_value('period_style'))
         self.__period_style.set_active(index)
 
         self.__auto_start_henkan_keywords.set_text(\
-            self.__config.get_value('auto_start_henkan_keywords',
-                                    ''.join(skk.AUTO_START_HENKAN_KEYWORDS)))
+            self.__config.get_value('auto_start_henkan_keywords'))
 
         self.__page_size.set_numeric(True)
         self.__page_size.set_range(7.0, 21.0)
         self.__page_size.set_value(\
-            int(self.__config.get_value('page_size',
-                                        skk.CandidateSelector.PAGE_SIZE)))
+            int(self.__config.get_value('page_size')))
         self.__pagination_start.set_numeric(True)
         self.__pagination_start.set_range(0.0, 7.0)
         self.__pagination_start.set_value(\
-            int(self.__config.get_value('pagination_start',
-                                        skk.CandidateSelector.PAGINATION_START)))
+            int(self.__config.get_value('pagination_start')))
         self.__show_annotation.set_active(\
-            self.__config.get_value('show_annotation', True))
+            self.__config.get_value('show_annotation'))
 
         renderer = gtk.CellRendererText()
         self.__rom_kana_rule.pack_start(renderer)
         self.__rom_kana_rule.set_attributes(renderer, text=0)
         index = get_index_by_value(\
             self.__rom_kana_rule,
-            self.__config.get_value('rom_kana_rule', skk.ROM_KANA_NORMAL))
+            self.__config.get_value('rom_kana_rule'))
         self.__rom_kana_rule.set_active(index)
 
         renderer = gtk.CellRendererText()
@@ -103,16 +112,23 @@ class PreferencesDialog:
         self.__initial_input_mode.set_attributes(renderer, text=0)
         index = get_index_by_value(\
             self.__initial_input_mode,
-            self.__config.get_value('initial_input_mode',
-                                    skk.INPUT_MODE_HIRAGANA))
+            self.__config.get_value('initial_input_mode'))
         self.__initial_input_mode.set_active(index)
 
+        self.__egg_like_newline.set_active(\
+            self.__config.get_value('egg_like_newline'))
+
         self.__usrdict.connect('file-set', self.__usrdict_file_set_cb)
-        self.__sysdict_file.connect('toggled', self.__sysdict_toggle_cb)
-        self.__sysdict_skkserv.connect('toggled', self.__sysdict_toggle_cb)
-        self.__sysdict.connect('file-set', self.__sysdict_file_set_cb)
+        self.__use_skkserv.connect('toggled', self.__use_skkserv_toggle_cb)
         self.__skkserv_host.connect('changed', self.__skkserv_host_changed_cb)
         self.__skkserv_port.connect('changed', self.__skkserv_port_changed_cb)
+        self.__sysdict.get_selection().connect_after('changed',
+                                                     self.__sysdict_selection_changed_cb)
+        self.__add_sysdict.connect('clicked', self.__add_sysdict_clicked_cb)
+        self.__remove_sysdict.connect('clicked', self.__remove_sysdict_clicked_cb)
+        self.__up_sysdict.connect('clicked', self.__up_sysdict_clicked_cb)
+        self.__down_sysdict.connect('clicked', self.__down_sysdict_clicked_cb)
+        self.__use_mmap.connect('toggled', self.__use_mmap_changed_cb)
         self.__period_style.connect('changed', self.__period_style_changed_cb)
         self.__auto_start_henkan_keywords.connect(\
             'changed', self.__auto_start_henkan_keywords_changed_cb)
@@ -121,34 +137,102 @@ class PreferencesDialog:
         self.__show_annotation.connect('toggled', self.__show_annotation_changed_cb)
         self.__rom_kana_rule.connect('changed', self.__rom_kana_rule_changed_cb)
         self.__initial_input_mode.connect('changed', self.__initial_input_mode_changed_cb)
-
-    def __sysdict_toggle_cb(self, widget):
-        sysdict_type = 'file' if self.__sysdict_file.get_active() else 'skkserv'
-        self.__config.set_value('sysdict_type', sysdict_type)
-        self.__set_sysdict_widgets_sensitivity(sysdict_type)
+        self.__egg_like_newline.connect('toggled', self.__egg_like_newline_changed_cb)
 
     def __set_sysdict_widgets_sensitivity(self, sysdict_type):
         if sysdict_type == 'file':
             self.__sysdict.set_sensitive(True)
+            self.__add_sysdict.set_sensitive(True)
             self.__skkserv_host.set_sensitive(False)
             self.__skkserv_port.set_sensitive(False)
+            self.__sysdict_entry_buttons_set_sensitivity(\
+                self.__sysdict.get_selection().get_selected()[1] is not None)
         else:
             self.__sysdict.set_sensitive(False)
+            self.__add_sysdict.set_sensitive(False)
             self.__skkserv_host.set_sensitive(True)
             self.__skkserv_port.set_sensitive(True)
+            self.__sysdict_entry_buttons_set_sensitivity(False)
 
-    def __sysdict_file_set_cb(self, widget):
-        self.__config.set_value('sysdict', widget.get_filename())
+    def __sysdict_entry_buttons_set_sensitivity(self, sensitive):
+        self.__remove_sysdict.set_sensitive(sensitive)
+        self.__up_sysdict.set_sensitive(sensitive)
+        self.__down_sysdict.set_sensitive(sensitive)
 
+    def __set_sysdict_from_model(self, model):
+        paths = list()
+        _iter = model.get_iter_root()
+        while _iter:
+            value, = model.get(_iter, 0)
+            paths.append(value)
+            _iter = model.iter_next(_iter)
+        self.__config.set_value('sysdict_paths', paths)
+        
+    # Callbacks
     def __usrdict_file_set_cb(self, widget):
         self.__config.set_value('usrdict', widget.get_filename())
         
+    def __use_skkserv_toggle_cb(self, widget):
+        sysdict_type = 'skkserv' if self.__use_skkserv.get_active() else 'file'
+        self.__config.set_value('sysdict_type', sysdict_type)
+        self.__set_sysdict_widgets_sensitivity(sysdict_type)
+
     def __skkserv_host_changed_cb(self, widget):
         self.__config.set_value('skkserv_host', widget.get_text())
 
     def __skkserv_port_changed_cb(self, widget):
         self.__config.set_value('skkserv_port', widget.get_text())
 
+    def __sysdict_selection_changed_cb(self, selection):
+        self.__sysdict_entry_buttons_set_sensitivity(\
+            selection.get_selected()[1] is not None)
+
+    def __add_sysdict_clicked_cb(self, widget):
+        chooser = gtk.FileChooserDialog(\
+            title=_("Open Dictionary File"),
+            parent=self.__dialog,
+            action=gtk.FILE_CHOOSER_ACTION_OPEN,
+            buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
+                     gtk.STOCK_OPEN, gtk.RESPONSE_OK))
+        if chooser.run() == gtk.RESPONSE_OK:
+            model = self.__sysdict.get_model()
+            path = chooser.get_filename()
+            _iter = model.get_iter_root()
+            while _iter:
+                _value, = model.get(_iter, 0)
+                if _value == path:
+                    break
+                _iter = model.iter_next(_iter)
+            if not _iter:
+                model.append((path,))
+                self.__set_sysdict_from_model(model)
+        chooser.hide()
+
+    def __remove_sysdict_clicked_cb(self, widget):
+        model, paths = self.__sysdict.get_selection().get_selected_rows()
+        for path in paths:
+            model.remove(model.get_iter(path))
+        self.__set_sysdict_from_model(model)
+
+    def __up_sysdict_clicked_cb(self, widget):
+        model, _iter = self.__sysdict.get_selection().get_selected()
+        if _iter:
+            path = model.get_path(_iter)
+            if path[0] > 0:
+                model.move_before(_iter, model.get_iter((path[0] - 1,)))
+                self.__set_sysdict_from_model(model)
+
+    def __down_sysdict_clicked_cb(self, widget):
+        model, _iter = self.__sysdict.get_selection().get_selected()
+        if _iter:
+            path = model.get_path(_iter)
+            if path[0] < len(model) - 1:
+                model.move_after(_iter, model.get_iter((path[0] + 1,)))
+                self.__set_sysdict_from_model(model)
+
+    def __use_mmap_changed_cb(self, widget):
+        self.__config.set_value('use_mmap', widget.get_active())
+
     def __period_style_changed_cb(self, widget):
         self.__config.set_value('period_style', widget.get_active())
 
@@ -174,8 +258,12 @@ class PreferencesDialog:
             val, = widget.get_model().get(_iter, 1)
             self.__config.set_value('initial_input_mode', val)
 
+    def __egg_like_newline_changed_cb(self, widget):
+        self.__config.set_value('egg_like_newline', widget.get_active())
+
     def run(self):
-        return self.__dialog.run()
+        if self.__dialog.run() == gtk.RESPONSE_OK:
+            self.__config.commit_all()
 
 def main():
     PreferencesDialog().run()
